## Copyright (C) Hugh Pumphrey (2005)
## License: GPL Version 2

## We need to fail with a graceful error message if netpbm is not
## installed, on Windows as well as unixes. 

##if(Sys.info()[1] == "Linux" print("Boo")

##(presumably Windows says "Windows") for if the way to check the file
##is different.  On unixes, we can do

## foo<-system("which anytopnm",TRUE)

## and see if foo is character(0) or a string like "/usr/bin/anytopnm"
## Windows may need something different.

## The netpbm package (from GnuWin32) is easy to install on Windows,
## but the shell scripts (including anytopnm) don't work. We'll have
## to do the image type stuff by hand. (or install sh and use that to
## run netpbm items which are really shell scripts)

## To run the netpbm commands on Windows requires shell(); system() doesn't 
## work. To run netpbm commands on Unix requires system(); shell() is not
## available -- it must be specific to R on Windows.

read.image <- function(filename,fformat=NULL,opts="",...){
  ## Function to read any image and return the data as a pixmap
  ## object.
  systype <- Sys.info()[1]

  if(systype == "Windows"){
    my.syscmd<-shell
    if(is.null(fformat)){
      fformat <- "png"
      print("No fformat supplied -- trying png." )
    }
    if(fformat=="auto"){
      print("Sorry, you can not use fformat=auto with windows - trying png")
      fformat <- "png"
    }
  }else{
    my.syscmd<-system
    if(is.null(fformat))
      fformat <- "auto"
  }
  
  tmpname <- tempfile()

  if(fformat == "tiff"){
    tcmd<-paste("tifftopnm ",opts,filename," > ", tmpname)
    print(tcmd)
    out <- my.syscmd(tcmd)
  }else if(fformat=="png"){
    out <- my.syscmd(paste("pngtopnm",opts, filename," > ",tmpname))
  }else if(fformat=="jpg"){
    out <- my.syscmd(paste("jpegtopnm",opts,filename," > ",tmpname))
  }else if(fformat=="gif"){
   out <-  my.syscmd(paste("giftopnm",opts,filename," > ",tmpname))
  }else if(fformat=="tst"){
    ## This is a test, to see if we get a sensible error if
    ## netpbm is not installed
    out <- my.syscmd(paste("rubbishtopnm",opts,filename," > ",tmpname))
  }else if(fformat=="bmp"){            
    out <- my.syscmd(paste("bmptopnm",opts,filename," > ",tmpname))
  }else if(fformat=="ppm" || fformat=="pbm" ||
           fformat=="pgm" || fformat=="pnm"){
    file.copy(filename,tmpname,overwrite=TRUE)
  }else if(fformat=="auto"){
    out <- my.syscmd(paste("anytopnm",opts,filename," > ",tmpname))
  }else{
    print(paste("Invalid fformat",fformat))	
    out <- -99
  }
  if(out !=0){

    print(paste("Error=",out))
    if(out > 300 || out < 0){
      print("Failed to read file. Please check if netpbm is installed.")
    }else{
      print("Failed to read file. Maybe you gave the wrong file format")
    }
    img<-NULL
  }else{ 
    img <- read.pnm(tmpname,...)
  }
  ## It might be more satisfactory to use a pipe, but this ...
  ## img <- read.pnm(pipe(paste("anytopnm",filename)))
  ## ... doesnt work.
  
  ## Delete temporary file
  unlink(tmpname)
  img
}


### write.pnm gives message:
## Warning message:
## connection is already open
## This is not a bug in write.image

write.image <- function(img,filename,fformat=NULL,opts="",...){
  systype <- Sys.info()[1]

  if(fformat=="auto"){
    print("Sorry, you can not use fformat=auto when writing. Using png")
    fformat <- "png"
  }
  if(is.null(fformat)){
    fformat <- "png"
    print("No fformat supplied -- trying png." )
  }  
  if(systype == "Windows"){
    my.syscmd<-shell	
  }else{
    my.syscmd<-system
  }
  

  tmpname <- tempfile()
  write.pnm(img,tmpname,...)
  if(fformat == "tiff"){
    out<-my.syscmd(paste("pnmtotiff",opts,tmpname, " > ", filename))
  }else if(fformat == "jpeg"){
    out<-my.syscmd(paste("pnmtojpeg",opts,tmpname, " > ", filename))
  }else if(fformat == "gif"){
    out<-my.syscmd(paste("ppmquant 255",tmpname," | ppmtogif  ",
                      opts," > ", filename))
  }else if(fformat == "bmp"){
    out<-my.syscmd(paste("ppmtobmp",opts,tmpname, " > ", filename))
  }else{
    if(fformat != "png"){
      print(paste("Unrecognised fformat",fformat," --  Writing as png."))
    }
    out<-my.syscmd(paste("pnmtopng",tmpname, " > ", filename))
  }


  if(out !=0){
    
    ## print(paste("Error=",out))
    if(out > 300 || out < 0){
      print("Failed to write file. Please check if netpbm is installed.")
    }else{
      print("Failed to write file. Maybe you gave the wrong file format")
    }
  }
  ## delete temporary file
  unlink(tmpname)
  out
}
